//
//  ComboBox.m
//  version 1.0
//
//  Created by silversik on 10. 6. 29..
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "ComboBox.h"

@implementation ComboCell

@synthesize textColor;
@synthesize hilightColor;
@synthesize separatorColor;

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
    if ((self = [super initWithStyle:style reuseIdentifier:reuseIdentifier])) {
        // Initialization code
        separatorColor = COLOR_SEPARATOR;
        
        [self.textLabel setBackgroundColor:[UIColor clearColor]];
		[self setSelectionStyle:UITableViewCellSelectionStyleNone];
    }
    return self;
}

- (void) setHighlighted:(BOOL)highlighted animated:(BOOL)animated {
    
	flagHighlighted = highlighted;
	[self setNeedsDisplay];
}


- (void) drawRect:(CGRect)rect {
	CGContextRef context = UIGraphicsGetCurrentContext();
	
    if (flagHighlighted) {
        if (hilightColor) {
             [self.textLabel setTextColor:hilightColor];
        }
       
    }
    else {
        if (textColor) {
            [self.textLabel setTextColor:textColor];
        }
        
    }


	CGContextSetStrokeColor(context, CGColorGetComponents([separatorColor CGColor]));
	CGContextSetLineWidth(context, 0.5);
	CGContextMoveToPoint(context, 10, self.frame.size.height - 0.5);
	CGContextAddLineToPoint(context, self.frame.size.width-10, self.frame.size.height - 0.5);
	CGContextStrokePath(context);
}

- (void)dealloc {
    [super dealloc];
}


@end


@implementation ComboBox

@synthesize index;
@synthesize itemArray;
@synthesize fCellHeight;
@synthesize backView;

-(id) initWithFrame:(CGRect)frame maxShowCount:(NSInteger)_maxShowCount {
	if(self = [super init])
	{
		// Initialization code
        index = 0;
		maxShowCount = _maxShowCount;
		fCellHeight = frame.size.height;
		textFont = [UIFont systemFontOfSize:SIZE_FONT];
		textColor = COLOR_DEFAULT;
        hilightColor = COLOR_HILIGHT;
        
		[self setFrame:frame];
		
		isOpen = NO;
		delegate = nil;
		
		backView = [[UIView alloc] initWithFrame:self.bounds];
		[backView setBackgroundColor:[UIColor whiteColor]];
		[backView.layer setCornerRadius:6.0f];
		[backView.layer setMasksToBounds:YES];
		[self addSubview:backView];
		[backView release];
		
		textLabel = [[UILabel alloc] initWithFrame:CGRectMake(10, 0, self.bounds.size.width-20, self.bounds.size.height)];
		[textLabel setFont:textFont];
		[textLabel setBackgroundColor:[UIColor clearColor]];
		[self addSubview:textLabel];
		[textLabel release];
		
		itemTable = [[UITableView alloc] initWithFrame:CGRectMake(0.0, frame.size.height, frame.size.width, frame.size.height*maxShowCount) style:UITableViewStylePlain];
		[itemTable setRowHeight:fCellHeight];
		[itemTable setSeparatorStyle:UITableViewCellSeparatorStyleSingleLineEtched];
		[itemTable.layer setCornerRadius:8.0f];
		[itemTable.layer setMasksToBounds:YES];
		[itemTable setDelegate:self];
		[itemTable setDataSource:self];
        [self addSubview:itemTable];
        [itemTable setAlpha:0.0];
	}
	
	return self;
}

#pragma mark -
- (void)addTarget:(id)target action:(SEL)_action {
	delegate = target;
	action = _action;
}

-(void) setItemArray:(NSMutableArray *)arr {
	itemArray = [arr retain];
	[textLabel setText:[itemArray objectAtIndex:0]];
}

-(NSInteger) count; {
	return [itemArray count];
}

-(void)insertItem:(NSString *)str {
	[itemArray addObject:str];

}

-(void)replaceItemAtIndex:(NSInteger)_index string:(NSString*)str {
	[itemArray replaceObjectAtIndex:_index withObject:str];
}

#pragma mark -
#pragma mark === Touch Delegate ===
- (void) touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {

	if (isOpen) {
		[self setFrame:CGRectMake(self.frame.origin.x, self.frame.origin.y, self.frame.size.width, backView.frame.size.height)];
	}
    else {
        int count;
        if( self.count == 0)			count = 1;
        if( self.count > maxShowCount)	count = maxShowCount;
        
        [self.superview bringSubviewToFront:self];
        
        [itemTable setFrame:CGRectMake(0.0, self.frame.size.height+2, self.frame.size.width, fCellHeight*self.count)];        
        [self setFrame:CGRectMake(self.frame.origin.x, self.frame.origin.y, self.frame.size.width, fCellHeight*(self.count+1))];
    }
    isOpen = !isOpen;
    
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:TIME_FADE];
    [itemTable setAlpha:isOpen];
    [UIView commitAnimations];
    
	return;
}

#pragma mark -
#pragma mark === TableView Delegate ===
- (NSInteger) tableView:(UITableView *)table numberOfRowsInSection:(NSInteger)section {
	return self.count;
}

- (UITableViewCell *) tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    NSString * cellID = @"ComboCell";
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellID];
	if (cell == nil)
	{
		cell = [[[ComboCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellID] autorelease];
        [cell.textLabel setFont:textFont];
    }
    
    [(ComboCell*)cell setTextColor:textColor];
    [(ComboCell*)cell setHilightColor:hilightColor];
    [(ComboCell*)cell setSeparatorColor:separatorColor];
    
    [cell.textLabel setText:[itemArray objectAtIndex:indexPath.row]];
	
	return cell;
}

- (void) tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {	
	index = indexPath.row;
	
	UITableViewCell * cell = [tableView cellForRowAtIndexPath:indexPath];
	[textLabel setText:cell.textLabel.text];
    
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:TIME_FADE];
    [itemTable setAlpha:0.0];
    [UIView commitAnimations];
    
	if (delegate) [delegate performSelector:action withObject:self];
    
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (void) setFCellHeight:(CGFloat)_fCellHeight {
    fCellHeight = _fCellHeight;
    [itemTable setRowHeight:fCellHeight];
}

#pragma mark -
#pragma mark === UI ===
- (void) setBackgroundColor:(UIColor *)color
{
	[backView	setBackgroundColor:color];
	[itemTable	setBackgroundColor:color];
}

- (void) setSeparatorColor:(UIColor*)color width:(CGFloat)width{
    separatorColor = color;
    
    [itemTable setSeparatorColor:separatorColor];
    [itemTable.layer setBorderColor:[separatorColor CGColor]];
    [itemTable.layer setBorderWidth:width];
    
    [backView.layer setBorderColor:[separatorColor CGColor]];
    [backView.layer setBorderWidth:width];
}

- (void) setTextColor:(UIColor *)_defaultColor hilightColor:(UIColor*)_hilightColor {
	textColor = _defaultColor;   
    hilightColor = _hilightColor;
	[textLabel setTextColor:_defaultColor];
    
}
- (void) setFont:(UIFont*)font {
	textFont = font;
	[textLabel setFont:font];
}

-(void) setButtonImage:(UIImage*)img rightMargin:(CGFloat)rightMargin {
	UIImageView * imgView = [[UIImageView alloc] initWithFrame:CGRectMake(textLabel.frame.size.width-img.size.width-rightMargin,
																		  (backView.bounds.size.height-img.size.height)/2,
																		  img.size.width,
																		  img.size.height)];
	[imgView setImage:img];
	[textLabel addSubview:imgView];
	[imgView release];
}

#pragma mark -
- (void)dealloc {
	[itemTable release];
	[itemArray release];
    [super dealloc];
}

@end
